﻿using CMS.DocumentEngine.Types;
using CMS.FileSystemStorage;
using CMS.Helpers;
using DancingGoat.Helpers;
using DancingGoat.Repositories;
using Microsoft.Azure.Search;
using Microsoft.Azure.Search.Models;
using Microsoft.Spatial;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace MVCDemo_TestDemoApp.Helpers
{
    class AzureSearchHelper
    {
        private static ISearchServiceClient _searchClient;
        private static ISearchIndexClient _indexClient;

        public static string errorMessage;

        static AzureSearchHelper()
        {
            try
            {
                string searchServiceName = ValidationHelper.GetString(ConfigurationManager.AppSettings["AzureSearchName"], "");
                string apiKey = ValidationHelper.GetString(ConfigurationManager.AppSettings["AzureSearchAPIKey"], "");

                // Create an HTTP reference to the Azure Search
                _searchClient = new SearchServiceClient(searchServiceName, new SearchCredentials(apiKey));
            }
            catch (Exception ex)
            {
                errorMessage = ex.Message.ToString();
            }
        }

        #region IndexActions

        public string CreateBlobDataSource()
        {
            // Create the Azure Search index based on the included schema
            try
            {
                string strReturn = "";

                // Create the data source
                _searchClient.DataSources.CreateOrUpdate(new DataSource("blobsdatasource", DataSourceType.AzureBlob, new DataSourceCredentials(ValidationHelper.GetString(ConfigurationManager.AppSettings["AzureStorageCredentials"], "")), new DataContainer("cmsstorage", "dancinggoat/media/sample")));

                strReturn = "Datasource created!";

                return strReturn;

            }
            catch (Exception ex)
            {
                return "There was an issue creating the data source: {0}\r\n" + ex.Message.ToString();
            }
        }

        public string CreateBlobIndex()
        {
            try
            {
                string strReturn = "";
                // Create the index definition
                var definition = new Index()
                {
                    Name = "blobsindex",
                    Fields = new[]
                    {
                    new Field ( "ID", DataType.String) { IsKey = true,  IsSearchable = false, IsFilterable = false, IsSortable = false, IsFacetable = false, IsRetrievable = true},
                    new Field ( "Content", DataType.String) { IsKey = false,  IsSearchable = true, IsFilterable = true, IsSortable = false, IsFacetable = false, IsRetrievable = true},
                    new Field ( "Name", DataType.String),
                    new Field ( "Path", DataType.String),
                    },
                };

                // Create the index
                _searchClient.Indexes.CreateOrUpdate(definition);
                strReturn = "Blobs index created!";

                return strReturn;

            }
            catch (Exception ex)
            {
                return "There was an issue creating the blobs index: {0}\r\n" + ex.Message.ToString();
            }
        }

        public string CreateBlobIndexer()
        {
            try
            {
                string strReturn = "";

                Indexer blobindexer = new Indexer();
                blobindexer.Name = "blobsindexer";
                blobindexer.DataSourceName = "blobsdatasource";
                blobindexer.TargetIndexName = "blobsindex";

                IndexingSchedule indexingschedule = new IndexingSchedule();
                indexingschedule.Interval = new TimeSpan(0, 5, 0);
                blobindexer.Schedule = indexingschedule;

                List<FieldMapping> fieldmappings = new List<FieldMapping>();
                fieldmappings.Add(new FieldMapping("metadata_storage_name", "name"));
                fieldmappings.Add(new FieldMapping("metadata_storage_path", "path"));
                blobindexer.FieldMappings = fieldmappings;

                _searchClient.Indexers.CreateOrUpdate(blobindexer);

                strReturn = "Blobs indexer created!";

                return strReturn;

            }
            catch (Exception ex)
            {
                return "There was an issue creating the blobs indexer: {0}\r\n" + ex.Message.ToString();
            }
        }

        public string DeleteBlobIndex()
        {
            try
            {
                string strReturn = "";
                // Delete the indexer
                _searchClient.Indexers.Delete("blobsindexer");

                // Delete the index
                _searchClient.Indexes.Delete("blobsindex");

                // Delete the data source
                _searchClient.DataSources.Delete("blobsdatasource");

                strReturn = "Blobs index deleted!";

                return strReturn;
            }
            catch (Exception ex)
            {
                return "There was an issue deleting the blobs index: {0}\r\n" + ex.Message.ToString();
            }
        }

        #endregion
    }
}
